@@ -8,6 +8,7 @@ import random
 import re
 import string
 import sys
+import time
 import zipfile
 from distutils.version import LooseVersion
 from urllib.request import urlopen, urlretrieve
@@ -61,6 +62,9 @@ class Patcher(object):
         self.executable_path = None
         prefix = secrets.token_hex(8)
 
+        if not os.path.exists(self.data_path):
+            os.makedirs(self.data_path, exist_ok=True)
+
         if not executable_path:
             self.executable_path = os.path.join(
                 self.data_path, "_".join([prefix, self.exe_name])
@@ -71,7 +75,7 @@ class Patcher(object):
                 if not executable_path[-4:] == ".exe":
                     executable_path += ".exe"
 
-        self.zip_path = os.path.join(self.data_path, self.zip_name)
+        self.zip_path = os.path.join(self.data_path, prefix)
 
         if not executable_path:
             self.executable_path = os.path.abspath(
@@ -146,7 +150,7 @@ class Patcher(object):
     def parse_exe_version(self):
         with io.open(self.executable_path, "rb") as f:
             for line in iter(lambda: f.readline(), b""):
-                match = re.search(br"platform_handle\x00content\x00([0-9.]*)", line)
+                match = re.search(rb"platform_handle\x00content\x00([0-9.]*)", line)
                 if match:
                     return LooseVersion(match[1].decode())
 
@@ -173,14 +177,12 @@ class Patcher(object):
         except (FileNotFoundError, OSError):
             pass
 
-        os.makedirs(os.path.dirname(self.zip_path), mode=0o755, exist_ok=True)
+        os.makedirs(self.zip_path, mode=0o755, exist_ok=True)
         with zipfile.ZipFile(fp, mode="r") as zf:
-            zf.extract(self.exe_name, os.path.dirname(self.zip_path))
-        os.rename(
-            os.path.join(self.data_path, self.exe_name),
-            self.executable_path
-        )
+            zf.extract(self.exe_name, self.zip_path)
+        os.rename(os.path.join(self.zip_path, self.exe_name), self.executable_path)
         os.remove(fp)
+        os.rmdir(self.zip_path)
         os.chmod(self.executable_path, 0o755)
         return self.executable_path
 
@@ -246,15 +248,29 @@ class Patcher(object):
         )
 
     def __del__(self):
-        try:
-            if not self._custom_exe_path:
-                # we will not delete custom exe paths.
-                # but this also voids support.
-                # downloading and patching makes sure you never use the same $cdc values, see patch_exe()
-                # after all, this program has a focus on detectability...
-                os.unlink(self.executable_path)
 
-        # except (OSError, RuntimeError, PermissionError):
-        #     pass
-        except:
-            raise
+        if self._custom_exe_path:
+            # if the driver binary is specified by user
+            # we assume it is important enough to not delete it
+            return
+        else:
+            timeout = 3  # stop trying after this many seconds
+            t = time.monotonic()
+            while True:
+                now = time.monotonic()
+                if now - t > timeout:
+                    # we don't want to wait until the end of time
+                    logger.debug(
+                        "could not unlink %s in time (%d seconds)"
+                        % (self.executable_path, timeout)
+                    )
+                    break
+                try:
+                    os.unlink(self.executable_path)
+                    logger.debug("successfully unlinked %s" % self.executable_path)
+                    break
+                except (OSError, RuntimeError, PermissionError):
+                    time.sleep(0.1)
+                    continue
+                except FileNotFoundError:
+                    break
